@@ -1537,6 +1537,16 @@ gdk_pixbuf__jpeg_image_save_to_callback (GdkPixbufSaveFunc   save_func,
 			       TRUE, NULL, save_func, user_data);
 }
 
+static gboolean
+gdk_pixbuf__jpeg_is_save_option_supported (const gchar *option_key)
+{
+        if (g_strcmp0 (option_key, "quality") == 0 ||
+            g_strcmp0 (option_key, "icc-profile") == 0)
+                return TRUE;
+
+        return FALSE;
+}
+
 #ifndef INCLUDE_jpeg
 #define MODULE_ENTRY(function) G_MODULE_EXPORT void function
 #else
@@ -1551,6 +1561,7 @@ MODULE_ENTRY (fill_vtable) (GdkPixbufModule *module)
 	module->load_increment = gdk_pixbuf__jpeg_image_load_increment;
 	module->save = gdk_pixbuf__jpeg_image_save;
 	module->save_to_callback = gdk_pixbuf__jpeg_image_save_to_callback;
+        module->is_save_option_supported = gdk_pixbuf__jpeg_is_save_option_supported;
 }
 
 MODULE_ENTRY (fill_info) (GdkPixbufFormat *info)
